/*
 * @brief	lwIP Filesystem implementation module
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "board.h"
#include "fs.h"
#include "dosfs.h"
#include "lpc43xx_dualcore_config.h"

#include "httpd_structs.h"

/** @defgroup EXAMPLE_DUALCORE_LWIP HTTP Server dual core example using LWIP ethernet stack
 * @ingroup EXAMPLES_DUALCORE_43XX
 * The lwIP Filesystem implementation module implements the file system functions
 * required for FAT file system.
 * @{
 */

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* Default html file */
const static char http_index_html[] =
	"<html><head><title>Congrats!</title></head>"
	"<body><h1>Welcome to our lwIP HTTP server!</h1>"
	"<p>This is a small test page, served by httpd of "
	"lwip.</p></body></html>";

/* Internal File descriptor structure */
struct file_ds {
	VOLINFO vi;
	uint8_t scratch[SECTOR_SIZE];
	FILEINFO fi;
	struct fs_file fs;

};

/*****************************************************************************
 * Private functions
 ****************************************************************************/

#ifdef OS_FREE_RTOS
#include "FreeRTOS.h"
#include "semphr.h"
static xSemaphoreHandle open_lock;
/* FreeRTOS mutex lock */
static int mutex_lock(xSemaphoreHandle *mx)
{
	if (xSemaphoreTake(*mx, 500) != pdTRUE) {
		return 1;
	}
	return 0;
}

/* FreeRTOS mutex unlock */
static void mutex_unlock(xSemaphoreHandle *mx)
{
	xSemaphoreGive(*mx);
}

/* FreeRTOS mutex init */
static int mutex_init(void)
{
	open_lock = xSemaphoreCreateMutex();
	return open_lock == NULL;
}

#elif defined(OS_UCOS_III)

#include "os.h"
#include "cpu.h"
#include "os_cfg_app.h"

static OS_MUTEX open_lock;

/* mutex lock function for ucos-iii */
static int mutex_lock(OS_MUTEX *mx)
{
	CPU_TS ts;
	OS_ERR os_err;
	OSMutexPend(mx, 500, OS_OPT_PEND_BLOCKING, &ts, &os_err);
	if (os_err != OS_ERR_NONE) {
		return 1;
	}

	return 0;
}

/* Mutex unlock function for ucos-iii */
static void mutex_unlock(OS_MUTEX *mx)
{
	OS_ERR os_err;
	OSMutexPost(mx, OS_OPT_POST_NONE, &os_err);
}

/* Mutex init function for ucos-iii */
static int mutex_init(void)
{
	OS_ERR os_err;
	OSMutexCreate(&open_lock, "DOS_FS_Mutex", &os_err);
	if (os_err == OS_ERR_NONE) {
		return 0;
	}
	else {
		return 1;
	}
}

#else
static int open_lock;
/* Mutex lock function for standalone */
static int mutex_lock(int *mx)
{
	return 0;
}

/* Mutex init for standalone */
static int mutex_init(void)
{
	open_lock = 1;
	return 0;
}

/* Mutex lock for standalone */
static void mutex_unlock(int *mx)
{}

#endif

/**
 * Generate the relevant HTTP headers for the given filename and write
 * them into the supplied buffer.
 */
static int
get_http_headers(const char *fName, char *buff)
{
	unsigned int iLoop;
	const char *pszExt = NULL;
	const char *hdrs[4];

	/* Ensure that we initialize the loop counter. */
	iLoop = 0;

	/* In all cases, the second header we send is the server identification
	   so set it here. */
	hdrs[1] = g_psHTTPHeaderStrings[HTTP_HDR_SERVER];

	/* Is this a normal file or the special case we use to send back the
	   default "404: Page not found" response? */
	if (( fName == NULL) || ( *fName == 0) ) {
		hdrs[0] = g_psHTTPHeaderStrings[HTTP_HDR_NOT_FOUND];
		hdrs[2] = g_psHTTPHeaderStrings[DEFAULT_404_HTML];
		goto end_fn;
	}
	/* We are dealing with a particular filename. Look for one other
	    special case.  We assume that any filename with "404" in it must be
	    indicative of a 404 server error whereas all other files require
	    the 200 OK header. */
	if (strstr(fName, "404")) {
		iLoop = HTTP_HDR_NOT_FOUND;
	}
	else if (strstr(fName, "400")) {
		iLoop = HTTP_HDR_BAD_REQUEST;
	}
	else if (strstr(fName, "501")) {
		iLoop = HTTP_HDR_NOT_IMPL;
	}
	else {
		iLoop = HTTP_HDR_OK;
	}
	hdrs[0] = g_psHTTPHeaderStrings[iLoop];

	/* Get a pointer to the file extension.  We find this by looking for the
	     last occurrence of "." in the filename passed. */
	pszExt = strrchr(fName, '.');

	/* Does the FileName passed have any file extension?  If not, we assume it
	     is a special-case URL used for control state notification and we do
	     not send any HTTP headers with the response. */
	if (pszExt == NULL) {
		return 0;
	}

	pszExt++;
	/* Now determine the content type and add the relevant header for that. */
	for (iLoop = 0; (iLoop < NUM_HTTP_HEADERS); iLoop++)
		/* Have we found a matching extension? */
		if (!strcmp(g_psHTTPHeaders[iLoop].extension, pszExt)) {
			hdrs[2] =
				g_psHTTPHeaderStrings[g_psHTTPHeaders[iLoop].headerIndex];
			break;
		}

	/* Did we find a matching extension? */
	if (iLoop == NUM_HTTP_HEADERS) {
		/* No - use the default, plain text file type. */
		hdrs[2] = g_psHTTPHeaderStrings[HTTP_HDR_DEFAULT_TYPE];
	}

end_fn:
	iLoop = strlen(hdrs[0]);
	strcpy(buff, hdrs[0]);
	strcat(buff, hdrs[1]);
	strcat(buff, hdrs[2]);
	return strlen(buff);
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Get HTTP header function
 * @param fName	:   Filename for which the header be generated
 * @param buff	:   buffer to which the generated header be copied
 * @return Number of bytes in header
 * The function will read the HTTP header from the file & returns with
 * No. of bytes read
 */
int GetHTTP_Header(const char *fName, char *buff)
{
	return get_http_headers(fName, buff);
}

/**
 * @brief	Open default HTTP file function
 * @return Pointer to File structure of default file on success
 *         NULL on failure
 * The function will open the default file. The default file will have the
 * HTTP content pointed by http_index_html variable.
 * This function will be called if the HTTP files are not present in
 * the file system.
 */
struct fs_file *fs_open_default(void) {
	int hlen;

	struct file_ds *fds;

	struct fs_file *fs;

	fds = malloc(sizeof(*fds));
	if (fds == NULL) {
		DEBUGSTR("Malloc Failure, Out of Memory!\r\n");
		return NULL;
	}
	memset(fds, 0, sizeof(*fds));
	fs = &fds->fs;
	fs->pextension = (void *) fds;	/* Store this for later use */
	hlen = get_http_headers("default.htm", (char *) fds->scratch);
	fs->data = (const char *) fds->scratch;
	memcpy((void *) &fs->data[hlen], (void *) http_index_html, sizeof(http_index_html) - 1);
	fs->len = hlen + sizeof(http_index_html) - 1;
	fs->index = fs->len;
	fs->http_header_included = 1;
	return fs;
}

/**
 * @brief	Open a file from the Filesystem function
 * @param name	:	Name of the file to be opened
 * @return Pointer to File structure on success
 *         NULL on failure
 * The function will open the file present on the file system.
 */
struct fs_file *fs_open(const char *name) {
	int hlen;

	struct file_ds *fds;

	struct fs_file *fs;

	static struct file_ds tmpds;

	if (mutex_init()) {
		LWIP_DEBUGF(HTTPD_DEBUG, ("DFS: ERROR: Mutex Init!\r\n"));
		return NULL;
	}

	if (mutex_lock(&open_lock)) {
		LWIP_DEBUGF(HTTPD_DEBUG, ("DFS: ERROR: Mutex Timeout!\r\n"));
		return NULL;
	}
	memset(&tmpds, 0, sizeof(tmpds));
	fds = &tmpds;

	if (DFS_GetVolInfo(0, fds->scratch, 0, &fds->vi)) {
		LWIP_DEBUGF(HTTPD_DEBUG, ("DFS: VOLUME: Read Error!\r\n"));
		mutex_unlock(&open_lock);
		return NULL;
	}

	if (DFS_OpenFile(&fds->vi, (uint8_t *) name, DFS_READ, fds->scratch, &fds->fi) != DFS_OK) {
		LWIP_DEBUGF(HTTPD_DEBUG, ("DFS: OPEN: File %s does not exist\r\n", name));
		mutex_unlock(&open_lock);
		return NULL;
	}

	fds = malloc(sizeof(*fds));
 	if (fds == NULL) {
		DEBUGSTR("Malloc Failure, Out of Memory!\r\n");
		mutex_unlock(&open_lock);
		return NULL;
	}
	memcpy(fds, &tmpds, sizeof(*fds));
	mutex_unlock(&open_lock);

	fs = &fds->fs;
	fs->pextension = (void *) fds;	/* Store this for later use */
	hlen = get_http_headers(name, (char *) fds->scratch);
	fs->data = (const char *) fds->scratch;
	fs->index = hlen;
	fs->len = fds->fi.filelen + hlen;
	fs->http_header_included = 1;
	return fs;
}

/**
 * @brief	Closes/Frees a previously opened file function
 * @param file	:	Pointer to File structure of opened file
 * @return None
 * The function will close the file & free the resources.
 */
void fs_close(struct fs_file *file)
{
	if(!file)
		return;
 	free(file->pextension);
}

/**
 * @brief	This reads the requested number of bytes from the file function
 * @param file	:	Pointer to File structure of opened file
 * @param	buffer :	pointer to memroy, where the data be stored
 * @param	count	: Number of bytes to be read from the file
 * @return Number of bytes successfully read
 * The function will read the specified number of bytes from the file.
 */
int fs_read(struct fs_file *file, char *buffer, int count)
{
	uint32_t i;
	struct file_ds *fds = (struct file_ds *) file->pextension;
	DFS_ReadFile(&fds->fi, fds->scratch, (uint8_t *) buffer, &i, count);
	file->index += i;
	return i;
}

/**
 * @brief	Get number of bytes yet to be read in a file function
 * @param file	:	Pointer to File structure of opened file
 * @return Number of byets yet to be read from the file
 * The function will return the number bytes yet to be read from the file.
 */
int fs_bytes_left(struct fs_file *file)
{
	return file->len - file->index;
}

#ifdef LWIP_DEBUG
/**
 * @brief	Assert function for debug
 * @param msg	:	Message to be printed
 * @param	line	:	Line number from which the assert was raised
 * @param	file	:	Filename from which the assert was raised
 * @return None
 * The function will assert.
 */
void assert_printf(char *msg, int line, char *file)
{
	DEBUGOUT("ASSERT: %s at %s:%d\r\n", msg, file, line);
}

/**
 * @brief	This is a dummy function, required for debug
 * @param eno	:	Error Number
 * @return Empty string
 */
const char *lwip_strerr(err_t eno)
{
	return "";
}

#endif

/**
 * @brief		Function to read data from filesystem buffer
 * @param	unit : Unit number of the filesystem
 * @param	buffer : Pointer to memory to which read data to be stored
 * @param	sector : Starting sector from which the data to be read
 * @param	count	:	Number of sections to be read
 * @return 0 on success, non zero on failure
 * This function is called by DOS Filesystem library to read sectors
 * from the filesystem.
 */
uint32_t DFS_ReadSector(uint8_t unit, uint8_t *buffer, uint32_t sector, uint32_t count)
{
	memcpy(buffer, (void *) (FAT12_DISK_IMAGE + (sector * SECTOR_SIZE)), count * SECTOR_SIZE);
	return 0;
}

/*
 * @brief		Function to write data to filesystem buffer
 * @param unit : Unit number of the filesystem
 * @param	buffer : Pointer to data which has to be written to filesystem
 * @param	sector : Starting sector to which the \a buffer data to be written
 * @param	count :		Number of sections to be written
 * @return	0 on success, non zero on failure
 * This function is called by DOS Filesystem library to write sectors
 * to the filesystem.
 */
uint32_t DFS_WriteSector(uint8_t unit, uint8_t *buffer, uint32_t sector, uint32_t count)
{
	memcpy((void *) (FAT12_DISK_IMAGE + (sector * SECTOR_SIZE)), buffer, count * SECTOR_SIZE);
	return 0;
}

/**
 * @}
 */
