/*
 * @brief Make your board becomes a USB speaker
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "AudioOutput.h"
#if defined(USB_DEVICE_ROM_DRIVER)
#include ".\Lib\usbd_adcuser.h"
#endif
/** LPCUSBlib Audio Class driver interface configuration and state information. This structure is
 *  passed to all Audio Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_Audio_Device_t Speaker_Audio_Interface = {
	.Config = {
		.StreamingInterfaceNumber = 1,

		.DataOUTEndpointNumber    = AUDIO_STREAM_EPNUM,
		.DataOUTEndpointSize      = AUDIO_STREAM_EPSIZE,
	},
};

/** Max Sample Frequency. */
#define AUDIO_MAX_SAMPLE_FREQ   48000
/** Current audio sampling frequency of the streaming audio endpoint. */
uint32_t CurrentAudioSampleFrequency = AUDIO_MAX_SAMPLE_FREQ;

/**
 * Audio API
 */
/** Audio max packet count. */
#define AUDIO_MAX_PC    10
PRAGMA_ALIGN_4
uint8_t audio_buffer[2048] ATTR_ALIGNED(4);
uint32_t audio_buffer_size = 0;
uint32_t audio_buffer_rd_index = 0;
uint32_t audio_buffer_wr_index = 0;
uint32_t audio_buffer_count = 0;
void Audio_Reset_Data_Buffer(void)
{
	audio_buffer_count = 0;
	audio_buffer_wr_index = 0;
	audio_buffer_rd_index = 0;
}

uint32_t Audio_Get_ISO_Buffer_Address(uint32_t last_packet_size)
{
	audio_buffer_wr_index += last_packet_size;
	audio_buffer_count += last_packet_size;
	if (audio_buffer_count > audio_buffer_size) {
		Audio_Reset_Data_Buffer();
	}
	if (audio_buffer_wr_index >= audio_buffer_size) {
		audio_buffer_wr_index -= audio_buffer_size;
	}
	return (uint32_t) &audio_buffer[audio_buffer_wr_index];
}

void Audio_Init(uint32_t samplefreq)
{
	Chip_I2S_Audio_Format_Type audio_Confg;
	audio_Confg.SampleRate = samplefreq;
	audio_Confg.ChannelNumber = 2;	// 1 is mono, 2 is stereo
	audio_Confg.WordWidth = 16;	// 8, 16 or 32 bits
	Board_I2C_Init(LPC_I2C0);
	Board_Audio_Init(LPC_I2S0, MCB_18XX_AUDIO_LINE_IN_SELECT);
	Chip_I2S_Init(LPC_I2S0);
	Chip_I2S_Config(LPC_I2S0, I2S_TX_MODE, &audio_Confg);
	Chip_I2S_Stop(LPC_I2S0, I2S_TX_MODE);
	Chip_I2S_Mute(LPC_I2S0, DISABLE);
	Chip_I2S_Start(LPC_I2S0, I2S_TX_MODE);
	Chip_I2S_Int_Cmd(LPC_I2S0, I2S_TX_MODE,    ENABLE, 4);
	NVIC_EnableIRQ(I2S0_IRQn);

	Audio_Reset_Data_Buffer();
	switch (samplefreq) {
	case 11025:
	case 22050:
	case 44100:
		audio_buffer_size = 1764;
		break;

	case 8000:
	case 16000:
	case 32000:
	case 48000:
	default:
		audio_buffer_size = samplefreq * 4 * AUDIO_MAX_PC / 1000;
		break;
	}
}

void Audio_DeInit(void)
{
	Chip_I2S_DeInit(LPC_I2S0);
	Chip_I2S_Int_Cmd(LPC_I2S0, I2S_TX_MODE,    DISABLE, 4);
	NVIC_DisableIRQ(I2S0_IRQn);
}

void I2S0_IRQHandler(void)
{
	uint32_t txlevel, i;
	txlevel = Chip_I2S_GetLevel(LPC_I2S0, I2S_TX_MODE);
	if (txlevel <= 4) {
		for (i = 0; i < 8 - txlevel; i++) {
			if (audio_buffer_count >= 4) {	/*has enough data */
				audio_buffer_count -= 4;
				Chip_I2S_Send(LPC_I2S0, *(uint32_t *) (audio_buffer + audio_buffer_rd_index));
				audio_buffer_rd_index += 4;
				if (audio_buffer_rd_index >= audio_buffer_size) {
					audio_buffer_rd_index -= audio_buffer_size;
				}
			}
			else {
				Chip_I2S_Send(LPC_I2S0, (uint32_t) NULL);
			}

			/*Skip some samples if buffer run writting too fast. */
			if ((audio_buffer_size != 0) && (audio_buffer_count >= (audio_buffer_size / 2))) {
				audio_buffer_count -= 4;
				audio_buffer_rd_index += 4;
				if (audio_buffer_rd_index >= audio_buffer_size) {
					audio_buffer_rd_index -= audio_buffer_size;
				}
			}
		}
	}
}

/** This callback function provides iso buffer address for HAL iso transfer processing.
 */
uint32_t CALLBACK_HAL_GetISOBufferAddress(const uint32_t EPNum, uint32_t *last_packet_size) {

	/* Check if this is audio stream endpoint */
	if (EPNum == AUDIO_STREAM_EPNUM) {
		return Audio_Get_ISO_Buffer_Address(*last_packet_size);
	}
	else {return 0; }
}

/** Main program entry point. This routine contains the overall program flow, including initial
 *  setup of all components and the main program loop.
 */
int main(void)
{
	SetupHardware();

#if defined(USB_DEVICE_ROM_DRIVER)
	UsbdAdc_Init();
#endif

	for (;; ) {
#if !defined(USB_DEVICE_ROM_DRIVER)
		Audio_Device_USBTask(&Speaker_Audio_Interface);
		USB_USBTask();
#endif
	}
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware(void)
{
	Board_Init();
	USB_Init();
}

#if !defined(USB_DEVICE_ROM_DRIVER)
/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= Audio_Device_ConfigureEndpoints(&Speaker_Audio_Interface);

	//	LEDs_SetAllLEDs(ConfigSuccess ? LEDMASK_USB_READY : LEDMASK_USB_ERROR);
}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void)
{
	Audio_Device_ProcessControlRequest(&Speaker_Audio_Interface);
}

void EVENT_Audio_Device_StreamStartStop(USB_ClassInfo_Audio_Device_t *const AudioInterfaceInfo)
{
	/* reset audio buffer */
	Audio_Reset_Data_Buffer();
}

/** Audio class driver callback for the setting and retrieval of streaming endpoint properties. This callback must be implemented
 *  in the user application to handle property manipulations on streaming audio endpoints.
 */
bool CALLBACK_Audio_Device_GetSetEndpointProperty(USB_ClassInfo_Audio_Device_t *const AudioInterfaceInfo,
												  const uint8_t EndpointProperty,
												  const uint8_t EndpointAddress,
												  const uint8_t EndpointControl,
												  uint16_t *const DataLength,
												  uint8_t *Data)
{
	/* Check the requested endpoint to see if a supported endpoint is being manipulated */
	if (EndpointAddress == (ENDPOINT_DIR_OUT | Speaker_Audio_Interface.Config.DataOUTEndpointNumber)) {
		/* Check the requested control to see if a supported control is being manipulated */
		if (EndpointControl == AUDIO_EPCONTROL_SamplingFreq) {
			switch (EndpointProperty) {
			case AUDIO_REQ_SetCurrent:
				/* Check if we are just testing for a valid property, or actually adjusting it */
				if (DataLength != NULL) {
					/* Set the new sampling frequency to the value given by the host */
					CurrentAudioSampleFrequency =
						(((uint32_t) Data[2] << 16) | ((uint32_t) Data[1] << 8) | (uint32_t) Data[0]);
					if (CurrentAudioSampleFrequency > AUDIO_MAX_SAMPLE_FREQ) {
						return false;
					}
					Audio_DeInit();
					Audio_Init(CurrentAudioSampleFrequency);
				}

				return true;

			case AUDIO_REQ_GetCurrent:
				/* Check if we are just testing for a valid property, or actually reading it */
				if (DataLength != NULL) {
					*DataLength = 3;

					Data[2] = (CurrentAudioSampleFrequency >> 16);
					Data[1] = (CurrentAudioSampleFrequency >> 8);
					Data[0] = (CurrentAudioSampleFrequency &  0xFF);
				}

				return true;
			}
		}
	}

	return false;
}

#else

#endif
