/*
 * @brief Definitions and declarations of Mouse device example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef _MOUSE_H_
#define _MOUSE_H_
		#include "board.h"
		#include "USB.h"

		#include <stdbool.h>
		#include <string.h>

		#include "Descriptors.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup Mouse_Device Mouse Device
 * @ingroup EXAMPLES_USB_18XX43XX
 * This example implements a HID class device with mouse report descriptors. 
 * Mouse movement reports are sent to the host by the device when buttons or 
 * the joystick is pressed on the development board.
 *
 * See readme.txt for building and running guide.
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *  - Keil MCB1857 and MCB4357 boards
 *  - NGX Xplorer 1830 and 4330 boards
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/** @defgroup Mouse_Device_Definition Main definitions
 * @{
 */
/** LED mask for the library LED driver, to indicate that the USB interface is not ready. */
		#define LEDMASK_USB_NOTREADY      LEDS_LED1

/** LED mask for the library LED driver, to indicate that the USB interface is enumerating. */
		#define LEDMASK_USB_ENUMERATING  (LEDS_LED2 | LEDS_LED3)

/** LED mask for the library LED driver, to indicate that the USB interface is ready. */
		#define LEDMASK_USB_READY        (LEDS_LED2 | LEDS_LED4)

/** LED mask for the library LED driver, to indicate that an error has occurred in the USB interface. */
		#define LEDMASK_USB_ERROR        (LEDS_LED1 | LEDS_LED3)

/**
 * @}
 */

/** @defgroup Mouse_Device_Functions Functions prototypes
 * @{
 */
void SetupHardware(void);

void EVENT_USB_Device_Connect(void);

void EVENT_USB_Device_Disconnect(void);

void EVENT_USB_Device_ConfigurationChanged(void);

void EVENT_USB_Device_ControlRequest(void);

void EVENT_USB_Device_StartOfFrame(void);

/** @brief	HID class driver callback function for the creation of HID reports to the host.
 *
 *  @param      HIDInterfaceInfo :  Pointer to the HID class interface configuration structure being referenced
 *  @param      ReportID :    Report ID requested by the host if non-zero, otherwise callback should set to the generated report ID
 *  @param      ReportType :  Type of the report to create, either HID_REPORT_ITEM_In or HID_REPORT_ITEM_Feature
 *  @param      ReportData :  Pointer to a buffer where the created report should be stored
 *  @param      ReportSize :  Number of bytes written in the report (or zero if no report is to be sent
 *
 *  @return Boolean true to force the sending of the report, false to let the library determine if it needs to be sent
 */
bool CALLBACK_HID_Device_CreateHIDReport(USB_ClassInfo_HID_Device_t *const HIDInterfaceInfo,
										 uint8_t *const ReportID,
										 const uint8_t ReportType,
										 void *ReportData,
										 uint16_t *const ReportSize);

/** @brief	HID class driver callback function for the processing of HID reports from the host.
 *
 *  @param      HIDInterfaceInfo :  Pointer to the HID class interface configuration structure being referenced
 *  @param      ReportID :    Report ID of the received report from the host
 *  @param      ReportType :  The type of report that the host has sent, either HID_REPORT_ITEM_Out or HID_REPORT_ITEM_Feature
 *  @param      ReportData :  Pointer to a buffer where the received report has been stored
 *  @param      ReportSize :  Size in bytes of the received HID report
 *	@return	Nothing
 */
void CALLBACK_HID_Device_ProcessHIDReport(USB_ClassInfo_HID_Device_t *const HIDInterfaceInfo,
										  const uint8_t ReportID,
										  const uint8_t ReportType,
										  const void *ReportData,
										  const uint16_t ReportSize);

/**
 * @}
 */

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif
