/*
 * @brief I2C example
 * This example show how to use the I2C interface
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include "chip.h"

/** @defgroup EXAMPLES_PERIPH_18XX43XX_I2C I2C example
 * @ingroup EXAMPLES_PERIPH_18XX43XX
 * The I2C example shows how to use I2C interface for communicating between
 * master and slave in POLLING/INTERRUPT mode.
 *
 * To use the example, connect a serial cable to the board's RS232/UART port and
 * start a terminal program to monitor the port. The terminal program on the host
 * PC should be setup for 115K8N1.
 *
 * After a welcome screen is displayed via UART, I2C0 and I2C1 are configured with
 * 100KHz speed. This example supports five modes.
 * 	- Before running one of the first four modes, user must have 2 boards connected
 * 	through I2C1 pins (make sure 2 boards share a common GND pin); after that, run
 * 	slave mode on one board and then run master mode on another board.
 * 		- If choosing master mode (press '1' for polling master or '2' for interrupt
 * 		master), three phases are generated: transmit an array of bytes (press '1');
 * 		receive an array of bytes and verify (press '2'); transmit, generate a repeat
 * 		start condition and receive an array of bytes from slave (press '3').
 * 		- If choosing slave mode (press '3' for polling slave or '4' for interrupt
 * 		slave), user can press '1' to generate three following phases: receive an array
 * 		of bytes and verify; transmit an array of bytes; receive, wait for repeat
 * 		start condition from master and transmit an array of bytes.
 * 	- The last mode (I2C TouchScreen - press '5' - only for Keil MCB boards): configures
 * 	I2C0 as polling master to communicate with STMPE811 and show the coordinates on
 * 	terminal when touching the screen.
 *
 * Build procedure:
 * <a href="http://www.lpcware.com/content/project/lpcopen-platform-nxp-lpc-microcontrollers/lpcopen-build-procedures/lpc18xx/43xx-lpco">LPCOpen 18xx/43xx build instructions</a>
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards (using UART0)
 *    - PE.13 : I2C1_SDA (X20 pin 5)
 *    - PE.15 : I2C1_SCL (X20 pin 3)
 *  - Keil MCB1857 and MCB4357 boards (using UART3)
 *    - PE.13 : I2C1_SDA
 *    - PE.15 : I2C1_SCL
 *  - NGX Xplorer 1830 and 4330 boards (no UART - users should modify the code to test this example) 
 *    - P2.3 : I2C1_SDA 
 *    - P2.4 : I2C1_SCL 
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

 #define I2CDEV_UDA1380_ADDR     (0x34 >> 1)
#define I2CDEV_STMPE811_ADDR    (0x82 >> 1)

/* Own Slave address in Slave I2C device */
#define I2CDEV_S_ADDR   (0x90 >> 1)

/* Max buffer length */
#define BUFFER_SIZE         0xFF

#define C_GLCD_H_SIZE   240
#define C_GLCD_V_SIZE   320

static char menu[] =
	"\n\rHello NXP Semiconductors \n\r"
	"I2C demo \n\r"
	"\n\rPress 1-5 to select I2C running mode or 'ESC' to terminate the demo:\n\r\n\r"
	"\t 1 - I2C Polling   Master\n\r"
	"\t 2 - I2C Interrupt Master\n\r"
	"\t 3 - I2C Polling   Slave \n\r"
	"\t 4 - I2C Interrupt Slave \n\r"
	"\t 5 - I2C TouchScreen \n\r"
	"\n\rNote: Master mode can only run combine with Slave mode and vice versa.";
static char menu1[] = "\n\r \t - Press x to exit this mode!\n\r";
static char menu2[] = "\n\rI2C demo terminated!";
static char menu3[] = "\t - Press c to continue...\n\r";

static volatile bool complete;
static volatile bool isMasterMode = true;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Print Welcome menu */
static void print_menu(void)
{
	DEBUGOUT(menu);
}

/* Initialize buffer */
static void Buffer_Init(uint8_t *buffer, uint8_t type)
{
	uint8_t i;

	if (type) {
		for (i = 0; i < BUFFER_SIZE; i++)
			buffer[i] = i;
	}
	else {
		for (i = 0; i < BUFFER_SIZE; i++)
			buffer[i] = 0;
	}
}

/* I2C master application in polling mode */
static void App_I2C_Polling_Master(void)
{
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr = 0xBA;
	uint8_t Master_Buf[BUFFER_SIZE];
	uint8_t *pdat;
	uint32_t i = 0;

	/* Enable Master I2C operation */
	Chip_I2C_Cmd(LPC_I2C1, I2C_MASTER_MODE, ENABLE);

	while (1) {
		DEBUGOUT("\n\r\n\rI2C Polling Master is running...\n\r");
		DEBUGOUT(menu1);
		DEBUGOUT(menu3);

		while (1) {
			buffer = DEBUGIN();
			if ((buffer == 'x') || (buffer == 'X')) {
				/* Disable Master I2C operation */
				Chip_I2C_Cmd(LPC_I2C1, I2C_MASTER_MODE, DISABLE);
				return;
			}
			else if ((buffer == 'c') || (buffer == 'C')) {
				break;
			}
		}
		/* Transmit */
		DEBUGOUT("\n\rPress '1' to transmit");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '1') {
				break;
			}
		}
		DEBUGOUT("\n\rTransmitting...");

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 1);

		transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = sizeof(Master_Buf);
		Chip_I2C_MasterTransmitData(LPC_I2C1, &transferMCfg, I2C_TRANSFER_POLLING);

		/* Receive */
		DEBUGOUT("\n\rPress '2' to receive");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '2') {
				break;
			}
		}
		DEBUGOUT("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = sizeof(Master_Buf);
		Chip_I2C_MasterReceiveData(LPC_I2C1, &transferMCfg, I2C_TRANSFER_POLLING);

		pdat = Master_Buf;
		// Verify
		for (i = 0; i < sizeof(Master_Buf); i++)
			if (*pdat++ != i) {
				DEBUGOUT("\n\rVerify error!");
				break;
			}
		if (i == sizeof(Master_Buf)) {
			DEBUGOUT("\n\rVerify successfully!");
		}

		/* Transmit and receive */
		DEBUGOUT("\n\rPress '3' to Transmit, then repeat start and receive...");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '3') {
				break;
			}
		}

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
		transferMCfg.tx_data = &regAddr;
		transferMCfg.tx_length = 1;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = sizeof(Master_Buf);
		Chip_I2C_MasterTransferData(LPC_I2C1, &transferMCfg, I2C_TRANSFER_POLLING);

		pdat = Master_Buf;
		// Verify
		for (i = 0; i < sizeof(Master_Buf); i++)
			if (*pdat++ != i) {
				DEBUGOUT("\n\rVerify error!");
				break;
			}
		if (i == sizeof(Master_Buf)) {
			DEBUGOUT("\n\rVerify successfully!");
		}
	}
}

/* I2C master application using interrupt */
static void App_I2C_Interrupt_Master(void)
{
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr = 0xBA;
	uint8_t Master_Buf[BUFFER_SIZE];
	uint8_t *pdat;
	uint32_t i = 0;

	/* Enable Master I2C operation */
	Chip_I2C_Cmd(LPC_I2C1, I2C_MASTER_MODE, ENABLE);

	while (1) {
		DEBUGOUT("\n\r\n\rI2C Interrupt Master is running...\n\r");
		DEBUGOUT(menu1);
		DEBUGOUT(menu3);

		while (1) {
			buffer = DEBUGIN();
			if ((buffer == 'x') || (buffer == 'X')) {
				/* Disable Master I2C operation */
				Chip_I2C_Cmd(LPC_I2C1, I2C_MASTER_MODE, DISABLE);
				return;
			}
			else if ((buffer == 'c') || (buffer == 'C')) {
				break;
			}
		}
		/* Transmit */
		DEBUGOUT("\n\rPress '1' to transmit");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '1') {
				break;
			}
		}
		DEBUGOUT("\n\rTransmitting...");

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 1);

		complete = false;
		transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = sizeof(Master_Buf);
		Chip_I2C_MasterTransmitData(LPC_I2C1, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		NVIC_EnableIRQ(I2C1_IRQn);
		while (!complete) ;

		/* Receive */
		DEBUGOUT("\n\rPress '2' to receive");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '2') {
				break;
			}
		}
		DEBUGOUT("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		complete = false;
		transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = sizeof(Master_Buf);
		Chip_I2C_MasterReceiveData(LPC_I2C1, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		NVIC_EnableIRQ(I2C1_IRQn);
		while (!complete) ;

		pdat = Master_Buf;
		// Verify
		for (i = 0; i < sizeof(Master_Buf); i++)
			if (*pdat++ != i) {
				DEBUGOUT("\n\rVerify error!");
				break;
			}
		if (i == sizeof(Master_Buf)) {
			DEBUGOUT("\n\rVerify successfully!");
		}

		/* Transmit and receive */
		DEBUGOUT("\n\rPress '3' to Transmit, then repeat start and receive...");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '3') {
				break;
			}
		}

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		complete = false;
		transferMCfg.sl_addr7bit = I2CDEV_S_ADDR;
		transferMCfg.tx_data = &regAddr;
		transferMCfg.tx_length = 1;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = sizeof(Master_Buf);
		Chip_I2C_MasterTransferData(LPC_I2C1, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		NVIC_EnableIRQ(I2C1_IRQn);
		while (!complete) ;

		pdat = Master_Buf;
		// Verify
		for (i = 0; i < sizeof(Master_Buf); i++)
			if (*pdat++ != i) {
				DEBUGOUT("\n\rVerify error!");
				break;
			}
		if (i == sizeof(Master_Buf)) {
			DEBUGOUT("\n\rVerify successfully!");
		}
	}
}

/* I2C slave application in polling mode */
static void App_I2C_Polling_Slave(void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF, regAddr = 0xBA;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff = 0;
	uint8_t *pdat;
	uint32_t i = 0;

	/* Enable Slave I2C operation */
	Chip_I2C_Cmd(LPC_I2C1, I2C_SLAVE_MODE, ENABLE);

	while (1) {
		DEBUGOUT("\n\r\n\rI2C Polling Slave is running...\n\r");
		DEBUGOUT(menu1);
		DEBUGOUT(menu3);
		while (1) {
			buffer = DEBUGIN();
			if ((buffer == 'x') || (buffer == 'X')) {
				/* Disable Slave I2C operation */
				Chip_I2C_Cmd(LPC_I2C1, I2C_SLAVE_MODE, DISABLE);
				return;
			}
			else if ((buffer == 'c') || (buffer == 'C')) {
				break;
			}
		}

		/* Receive */
		DEBUGOUT("\n\rPress '1' to Start");
		while (1) {
			buffer = DEBUGIN();
			if (buffer == '1') {
				break;
			}
		}
		DEBUGOUT("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = sizeof(Slave_Buf);
		Chip_I2C_SlaveReceiveData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_POLLING);

		pdat = Slave_Buf;
		// Verify
		for (i = 0; i < sizeof(Slave_Buf); i++)
			if (*pdat++ != i) {
				DEBUGOUT("\n\rVerify error!");
				break;
			}
		if (i == sizeof(Slave_Buf)) {
			DEBUGOUT("\n\rVerify successfully!");
		}

		/* Transmit */
		DEBUGOUT("\n\rTransmitting...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = sizeof(Slave_Buf);
		Chip_I2C_SlaveTransmitData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_POLLING);

		/* Receive and transmit */
		DEBUGOUT("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		transferSCfg.rx_data = &S_buff;
		transferSCfg.rx_length = 1;
		Chip_I2C_SlaveReceiveData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_POLLING);
		if (S_buff == regAddr) {
			transferSCfg.tx_data = Slave_Buf;
			transferSCfg.tx_length = sizeof(Slave_Buf);
			Chip_I2C_SlaveTransmitData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_POLLING);
		}
	}
}

/* I2C slave application using interrupt */
static void App_I2C_Interrupt_Slave(void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF, regAddr = 0xBA;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff = 0;
	uint8_t *pdat;
	uint32_t i = 0;

	/* Enable Slave I2C operation */
	Chip_I2C_Cmd(LPC_I2C1, I2C_SLAVE_MODE, ENABLE);

	while (1) {
		DEBUGOUT("\n\r\n\rI2C Interrupt Slave is running...\n\r");
		DEBUGOUT(menu1);
		DEBUGOUT(menu3);
		while (1) {
			buffer = DEBUGIN();
			if ((buffer == 'x') || (buffer == 'X')) {
				/* Disable Slave I2C operation */
				Chip_I2C_Cmd(LPC_I2C1, I2C_SLAVE_MODE, DISABLE);
				return;
			}
			else if ((buffer == 'c') || (buffer == 'C')) {
				break;
			}
		}
		/* Receive */
		DEBUGOUT("\n\rPress '1' to Start");

		while (1) {
			buffer = DEBUGIN();
			if (buffer == '1') {
				break;
			}
		}
		DEBUGOUT("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = false;
		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = sizeof(Slave_Buf);
		Chip_I2C_SlaveReceiveData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		NVIC_EnableIRQ(I2C1_IRQn);
		while (!complete) ;

		pdat = Slave_Buf;
		// Verify
		for (i = 0; i < sizeof(Slave_Buf); i++)
			if (*pdat++ != i) {
				DEBUGOUT("\n\rVerify error!");
				break;
			}
		if (i == sizeof(Slave_Buf)) {
			DEBUGOUT("\n\rVerify successfully!");
		}

		/* Transmit */
		DEBUGOUT("\n\rTransmitting...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		complete = false;
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = sizeof(Slave_Buf);
		Chip_I2C_SlaveTransmitData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		NVIC_EnableIRQ(I2C1_IRQn);
		while (!complete) ;

		/* Receive and transmit */
		DEBUGOUT("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		complete = false;
		transferSCfg.rx_data = &S_buff;
		transferSCfg.rx_length = 1;
		Chip_I2C_SlaveReceiveData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		NVIC_EnableIRQ(I2C1_IRQn);
		while (!complete) ;

		if (S_buff == regAddr) {
			complete = false;
			transferSCfg.tx_data = Slave_Buf;
			transferSCfg.tx_length = sizeof(Slave_Buf);
			Chip_I2C_SlaveTransmitData(LPC_I2C1, &transferSCfg, I2C_TRANSFER_INTERRUPT);
			NVIC_EnableIRQ(I2C1_IRQn);
			while (!complete) ;
		}
	}
}

/* I2C touch screen mode application */
static void App_I2C_TouchScreen(void)
{
#if defined(BOARD_KEIL_MCB_18574357)
	int16_t x, y;
	FlagStatus exitflag = RESET;
	uint8_t buffer = 0;

	/* Enable Master I2C operation */
	Init_Touch_Controller();
	DEBUGOUT(menu1);

	while (exitflag == RESET) {

		buffer = DEBUGIN();
		if ((buffer == 'x') || (buffer == 'X')) {
			exitflag = SET;
		}

		if (GetTouchPos(&x, &y)) {
			DEBUGOUT("\n\r\n\rX = %d", x);
			DEBUGOUT("\n\rY = %d", y);
		}
	}

	/* Disable Master I2C operation */
	Chip_I2C_Cmd(LPC_I2C0, I2C_MASTER_MODE, DISABLE);
#endif
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	I2C Interrupt Handler
 * @return	None
 */
void I2C1_IRQHandler(void)
{
	if (isMasterMode) {	/* Master Mode */
		Chip_I2C_Interrupt_MasterHandler(LPC_I2C1);
		if (Chip_I2C_Interrupt_MasterTransferComplete(LPC_I2C1)) {
			complete = true;
			NVIC_DisableIRQ(I2C1_IRQn);
		}
	}
	else {	/* Slave Mode */
		Chip_I2C_Interrupt_SlaveHandler(LPC_I2C1);
		if (Chip_I2C_Interrupt_SlaveTransferComplete(LPC_I2C1)) {
			complete = true;
			NVIC_DisableIRQ(I2C1_IRQn);
		}
	}
}

/**
 * @brief	Main program body
 * @return	int
 */
int main(void)
{
	I2C_OWNSLAVEADDR_CFG_Type OwnSlavAdr;
	FlagStatus exitflag;
	uint8_t buffer = 0xFF;

	Board_Init();
	Board_I2C_Init(LPC_I2C0);
	Board_I2C_Init(LPC_I2C1);

	/* print welcome screen */
	print_menu();

	/* Init I2C1 */
	Chip_I2C_Init(LPC_I2C1);
	Chip_I2C_SetClockRate(LPC_I2C1, 100000);

	/* Set  Own slave address for I2C1 device */
	OwnSlavAdr.GeneralCallState = DISABLE;
	OwnSlavAdr.SlaveAddrChannel = 0;
	OwnSlavAdr.SlaveAddrMaskValue = 0xFF;
	OwnSlavAdr.SlaveAddr_7bit = I2CDEV_S_ADDR;
	Chip_I2C_SetOwnSlaveAddr(LPC_I2C1, &OwnSlavAdr);

	exitflag = RESET;
	/* Read some data from the buffer */
	while (exitflag == RESET) {
		buffer = DEBUGIN();

		switch (buffer) {
		case '1':		/* Master Polling Mode */
			App_I2C_Polling_Master();
			print_menu();
			break;

		case '2':		/* Master Interrupt Mode */
			isMasterMode = true;
			App_I2C_Interrupt_Master();
			print_menu();
			break;

		case '3':		/* Slave Polling Mode */
			App_I2C_Polling_Slave();
			print_menu();
			break;

		case '4':		/* Slave Interrupt Mode */
			isMasterMode = false;
			App_I2C_Interrupt_Slave();
			print_menu();
			break;

		case '5':		/* Touch Screen */
			App_I2C_TouchScreen();
			print_menu();
			break;

		case 27:
			/* ESC key, set exit flag */
			DEBUGOUT(menu2);
			exitflag = SET;
			break;

		default:
			break;
		}
	}

	Chip_I2C_DeInit(LPC_I2C0);
	Chip_I2C_DeInit(LPC_I2C1);

	return 0;
}

/**
 * @}
 */
