/*
 * @brief I2S example
 * This example show how to use the I2S in 3 modes : Polling, Interrupt and DMA
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"

/** @defgroup EXAMPLES_PERIPH_18XX43XX_I2S I2S example
 * @ingroup EXAMPLES_PERIPH_18XX43XX
 * The I2S example shows how to configure I2S and UDA1380 to receive audio signal and
 * play back with three modes: polling, interrupt, and DMA.
 *
 * To use the example, connect a serial cable to the board's RS232/UART port and
 * start a terminal program to monitor the port.  The terminal program on the host
 * PC should be setup for 115K8N1.
 * Connect the computer line out to board line-in (using 3.5mm male-to-male cable),
 * plug the headphone/speaker into board line-out. Change modes by selecting the
 * option from the terminal. Play audio on computer and listen from target board!
 *
 * Build procedure:
 * <a href="http://www.lpcware.com/content/project/lpcopen-platform-nxp-lpc-microcontrollers/lpcopen-build-procedures/lpc18xx/43xx-lpco">LPCOpen 18xx/43xx build instructions</a>
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *  - Keil MCB1857 and MCB4357 boards
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#define BUFFER_FULL 0
#define BUFFER_EMPTY 1
#define BUFFER_AVAILABLE 2

typedef struct ring_buff {
	uint32_t buffer[256];
	uint8_t read_index;
	uint8_t write_index;
} Ring_Buffer_t;

static char WelcomeMenu[] = "\n\rHello NXP Semiconductors \n\r"
							"I2S DEMO : Connect audio headphone out from computer to line-in on tested board to get audio signal\n\r"
							"Please press \'1\' to test Polling mode\n\r"
							"Please press \'2\' to test Interrupt mode\n\r"
							"Please press \'3\' to test DMA mode\n\r"
							"Please press \'x\' to exit test mode\n\r"
							"Please press \'m\' to DISABLE/ENABLE mute\n\r";

static Ring_Buffer_t ring_buffer;

static uint8_t send_flag;
static uint8_t channelTC;
static uint8_t dmaChannelNum_I2S_Tx, dmaChannelNum_I2S_Rx;
static uint8_t dma_send_receive;

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Get status of the ring buffer */
static uint8_t ring_buff_get_status(Ring_Buffer_t *ring_buff)
{
	if (ring_buff->read_index == ring_buff->write_index) {
		return BUFFER_EMPTY;
	}
	else if (ring_buff->read_index == (ring_buff->write_index) + 1) {
		return BUFFER_FULL;
	}
	else {return BUFFER_AVAILABLE; }
}

/* Interrupt routine for I2S example */
static void App_Interrupt_Test(void)
{
	uint8_t bufferUART, continue_Flag = 1;
	FunctionalState mute_status = ENABLE;
	DEBUGOUT("I2S Interrupt mode\n\r");
	Chip_I2S_Int_Cmd(LPC_I2S0, I2S_RX_MODE,    ENABLE, 4);
	Chip_I2S_Int_Cmd(LPC_I2S0, I2S_TX_MODE,    ENABLE, 4);
	NVIC_EnableIRQ(I2S0_IRQn);
	while (continue_Flag) {
		bufferUART = 0xFF;
		bufferUART = DEBUGIN();
		switch (bufferUART) {
		case 'x':
			continue_Flag = 0;
			Chip_I2S_Int_Cmd(LPC_I2S0, I2S_RX_MODE,    DISABLE, 4);
			NVIC_DisableIRQ(I2S0_IRQn);
			DEBUGOUT(WelcomeMenu);
			break;

		case 'm':
			Chip_I2S_Mute(LPC_I2S0, mute_status);
			mute_status = (FunctionalState) !mute_status;
			if (!mute_status) {
				DEBUGOUT("MUTE ENABLE\n\r");
			}
			else {DEBUGOUT("MUTE DISABLE\n\r"); }
			break;

		default:
			break;
		}
	}
}

/* Polling routine for I2S example */
static void App_Polling_Test(void)
{
	uint32_t polling_data = 0;
	uint8_t bufferUART, continue_Flag = 1;
	FunctionalState mute_status = ENABLE;
	DEBUGOUT("I2S Polling mode\n\r");
	while (continue_Flag) {
		bufferUART = 0xFF;
		bufferUART = DEBUGIN();
		switch (bufferUART) {
		case 'x':
			continue_Flag = 0;
			DEBUGOUT(WelcomeMenu);
			break;

		case 'm':
			Chip_I2S_Mute(LPC_I2S0, mute_status);
			mute_status = (FunctionalState) !mute_status;
			if (!mute_status) {
				DEBUGOUT("MUTE ENABLE\n\r");
			}
			else {DEBUGOUT("MUTE DISABLE\n\r"); }
			break;

		default:
			break;
		}

		if (Chip_I2S_GetLevel(LPC_I2S0, I2S_RX_MODE) > 0) {
			polling_data = Chip_I2S_Receive(LPC_I2S0);
			send_flag = 1;
		}
		if ((Chip_I2S_GetLevel(LPC_I2S0, I2S_TX_MODE) < 4) && (send_flag == 1)) {
			Chip_I2S_Send(LPC_I2S0, polling_data);
			send_flag = 0;
		}
	}
}

/* DMA routine for I2S example */
static void App_DMA_Test(void)
{
	uint8_t continue_Flag = 1, bufferUART = 0xFF;
	FunctionalState mute_status = ENABLE;
	Chip_I2S_DMA_Cmd(LPC_I2S0, I2S_TX_MODE, I2S_DMA_REQUEST_NUMBER_1, ENABLE, 4);
	Chip_I2S_DMA_Cmd(LPC_I2S0, I2S_RX_MODE, I2S_DMA_REQUEST_NUMBER_2, ENABLE, 4);
	/* Initialize GPDMA controller */
	Chip_GPDMA_Init();
	/* Setting GPDMA interrupt */
	NVIC_DisableIRQ(DMA_IRQn);
	NVIC_SetPriority(DMA_IRQn, ((0x01 << 3) | 0x01));
	NVIC_EnableIRQ(DMA_IRQn);

	dmaChannelNum_I2S_Rx = Chip_DMA_GetFreeChannel(GPDMA_CONN_I2S_Rx_Channel_1);

	Chip_DMA_Transfer(dmaChannelNum_I2S_Rx,
					  GPDMA_CONN_I2S_Rx_Channel_1,
					  GPDMA_CONN_I2S_Tx_Channel_0,
					  GPDMA_TRANSFERTYPE_P2P_CONTROLLER_SrcPERIPHERAL,
					  1);

	DEBUGOUT("I2S DMA mode\n\r");
	while (continue_Flag) {
		bufferUART = 0xFF;
		bufferUART = DEBUGIN();
		switch (bufferUART) {
		case 'x':
			continue_Flag = 0;
			Chip_I2S_DMA_Cmd(LPC_I2S0, I2S_RX_MODE, 2, DISABLE, 1);
			Chip_I2S_DMA_Cmd(LPC_I2S0, I2S_TX_MODE, 1, DISABLE, 1);

			Chip_DMA_Stop(dmaChannelNum_I2S_Rx);
			NVIC_DisableIRQ(DMA_IRQn);
			DEBUGOUT(WelcomeMenu);
			break;

		case 'm':
			Chip_I2S_Mute(LPC_I2S0, mute_status);
			mute_status = (FunctionalState) !mute_status;
			if (!mute_status) {
				DEBUGOUT("MUTE ENABLE\n\r");
			}
			else {DEBUGOUT("MUTE DISABLE\n\r"); }
			break;

		default:
			break;
		}
	}
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	DMA interrupt handler sub-routine
 * @return	Nothing
 */
void DMA_IRQHandler(void)
{
	if (dma_send_receive == 1) {
		if (Chip_DMA_Interrupt(dmaChannelNum_I2S_Rx) == SUCCESS) {
			channelTC++;
		}
		else {
			/* Process error here */
		}
	}
	else {
		if (Chip_DMA_Interrupt(dmaChannelNum_I2S_Tx) == SUCCESS) {
			channelTC++;
		}
		else {
			/* Process error here */
		}
	}
}

/**
 * @brief	I2S0 interrupt handler sub-routine
 * @return	Nothing
 */
void I2S0_IRQHandler(void)
{
	while ((ring_buff_get_status(&ring_buffer) != BUFFER_FULL) && (Chip_I2S_GetLevel(LPC_I2S0, I2S_RX_MODE) > 0)) {
		ring_buffer.buffer[ring_buffer.write_index++] = Chip_I2S_Receive(LPC_I2S0);
	}
	while ((ring_buff_get_status(&ring_buffer) != BUFFER_EMPTY) && (Chip_I2S_GetLevel(LPC_I2S0, I2S_TX_MODE) < 8)) {
		Chip_I2S_Send(LPC_I2S0, ring_buffer.buffer[ring_buffer.read_index++]);
	}
}

/**
 * @brief  Main routine for I2S example
 * @return Nothing
 */
int main(void)
{

	Chip_I2S_Audio_Format_Type audio_Confg;
	uint8_t bufferUART, continue_Flag = 1;
	audio_Confg.SampleRate = 48000;
	/* Select audio data is 2 channels (1 is mono, 2 is stereo) */
	audio_Confg.ChannelNumber = 2;
	/* Select audio data is 16 bits */
	audio_Confg.WordWidth = 16;

	Board_Init();
#if defined( __GNUC__ )
	__sys_write(0, "", 0);
#endif
	Board_I2C_Init(LPC_I2C0);
	DEBUGOUT(WelcomeMenu);
	Board_Audio_Init(LPC_I2S0, MCB_18XX_AUDIO_LINE_IN_SELECT);

	Chip_I2S_Init(LPC_I2S0);
	Chip_I2S_Config(LPC_I2S0, I2S_RX_MODE, &audio_Confg);
	Chip_I2S_Config(LPC_I2S0, I2S_TX_MODE, &audio_Confg);

	Chip_I2S_Stop(LPC_I2S0, I2S_TX_MODE);
	Chip_I2S_Mute(LPC_I2S0, DISABLE);
	Chip_I2S_Start(LPC_I2S0, I2S_TX_MODE);
	send_flag = 0;
	while (continue_Flag) {
		bufferUART = 0xFF;
		bufferUART = DEBUGIN();
		switch (bufferUART) {
		case '1':
			App_Polling_Test();
			break;

		case '2':
			App_Interrupt_Test();
			break;

		case '3':
			App_DMA_Test();
			break;

		case 'x':
			continue_Flag = 0;
			DEBUGOUT("Thanks for using\n\r");
			break;

		default:
			break;
		}
	}
}

/**
 * @}
 */
