/*
 * @brief SD/MMC example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include "chip.h"
#include "fatfs\diskio.h"
#include "fatfs\rtc.h"
#include "fs_mci.h"
#include "fatfs\ff.h"

/** @defgroup EXAMPLES_PERIPH_18XX43XX_SDMMC SDMMC example
 * @ingroup EXAMPLES_PERIPH_18XX43XX
 * This example describes how to use the SD/MMC interface with a SD card
 * and a FATFS filesystem.
 *
 * To use the example, plug a SD card (Hitex A4 board) or microSD card (NGX or Keil
 * boards) and connect a serial cable to the board's RS232/UART port start a terminal 
 * program to monitor the port.  The terminal program on the host PC should be setup 
 * for 115K8N1. 
 *
 * Build procedure:
 * <a href="http://www.lpcware.com/content/project/lpcopen-platform-nxp-lpc-microcontrollers/lpcopen-build-procedures/lpc18xx/43xx-lpco">LPCOpen 18xx/43xx build instructions</a>
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *     - Close pin 1-2 of JP9
 *     - Open all SV3, SV6
 *     - Close all SV12
 *     - Uart1 terminal port: SV11-p2(U1_RXD) connect to SV1-p7, SV11-p4 (U1_TXD) connect to SV1-p5
 *  - Keil MCB1857 and MCB4357 boards (debug output terminal on Uart0)
 *  - NGX LPC1830 LPC4330 (no debug output terminal)
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#if _USE_XSTRFUNC == 0
#include <string.h>
#define xstrlen(x)      strlen(x)
#define xstrcpy(x, y)    strcpy(x, y)
#define xmemset(x, y, z)  memset(x, y, z)
#define xstrchr(x, y)    strchr(x, y)
#endif

#ifdef BOARD_HITEX_EVA_18504350
#define debugstr(str)  board_uart_out_string(str)
#else
#define debugstr(str)  DEBUGSTR(str)
#endif

/* buffer size (in byte) for R/W operations */
#define BUFFER_SIZE     4096

static FATFS Fatfs;	/* File system object */
static FIL Fil;	/* File object */

#ifdef __IAR_SYSTEMS_ICC__
#pragma data_alignment=4
static BYTE Buff[BUFFER_SIZE];		/* Working buffer */
#else
static BYTE Buff[BUFFER_SIZE] __attribute__ ((aligned(4)));	/* Working buffer */
#endif

static volatile UINT Timer = 0;		/* Performance timer (1kHz increment) */
static volatile int32_t sdio_wait_exit = 0;
static volatile uint32_t u32Milliseconds;
static volatile uint32_t timerCntms = 0;

#define TIME_INTERVAL  1/* 1ms */

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/* SDMMC card info structure */
mci_card_struct sdcardinfo;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Delay callback for timed SDIF/SDMMC functions */
static void sdmmc_waitms(uint32_t time)
{
	/* In an RTOS, the thread would sleep allowing other threads to run. For standalone operation, we just spin on a timer */
	/*  TIM_Waitus(time * 1000); */
	timerCntms = time;
	while (timerCntms) {}
}

/**
 * @brief	Sets up the SD event driven wakeup
 * @param	bits : Status bits to poll for command completion
 * @return	Nothing
 */
static void sdmmc_setup_wakeup(uint32_t bits)
{
	/* Wait for IRQ - for an RTOS, you would pend on an event here with a IRQ based wakeup. */
	NVIC_ClearPendingIRQ(SDIO_IRQn);
	sdio_wait_exit = 0;
	Chip_SDMMC_SetIntMask(bits);
	NVIC_EnableIRQ(SDIO_IRQn);
}

/**
 * @brief	A better wait callback for SDMMC driven by the IRQ flag
 * @param	bits : Status bits to poll for command completion
 * @return	0 on success, or failure condition (-1)
 */
static uint32_t sdmmc_irq_driven_wait(void)
{
	uint32_t status;

	/* Wait for event, would be nice to have a timeout, but keep it  simple */
	while (sdio_wait_exit == 0) {}

	/* Get status and clear interrupts */
	status = Chip_SDMMC_GetIntStatus();

	return status;
}

/* Initialize the Repetitive Interrupt Timer at 1s */
static void App_Timer_Init(void)
{
	Chip_RIT_Init();
	/* Configure time_interval for RIT
	 * In this case: time_interval = 1000 ms = 1s
	 * So, RIT will generate interrupt each 1s
	 */
	Chip_RIT_SetTimerInterval(TIME_INTERVAL);
}

/* Initialize SD/MMC */
static void App_SDMMC_Init()
{
	memset(&sdcardinfo, 0, sizeof(sdcardinfo));
	sdcardinfo.evsetup_cb = sdmmc_setup_wakeup;
	sdcardinfo.waitfunc_cb = sdmmc_irq_driven_wait;
	sdcardinfo.msdelay_func = sdmmc_waitms;

	/*  SD/MMC initialization */
	Board_SDMMC_Init();

	/* The SDIO driver needs to know the SDIO clock rate */
	Chip_Clock_EnableOpts(CLK_MX_SDIO, true, true, 1);
	sdcardinfo.clk_rate = Chip_Clock_GetRate(CLK_MX_SDIO);
	Chip_SDMMC_Init();
}

#ifdef BOARD_HITEX_EVA_18504350
/* Initialize the UART for debugging */
static void board_uart_debug_init(void)
{
	/*  Hitex EVA A4 is sharing Uart port 0 and SDIO pin so it is needed to use Uart port 1 */
	Board_UART_Init(LPC_UART1);
	Chip_UART_SetBaud(LPC_UART1, 115200);
	Chip_UART_ConfigData(LPC_UART1, UART_DATABIT_8, UART_PARITY_NONE, UART_STOPBIT_1);	/*  default 8-N-1 */

	/*  Enable UART Transmit */
	Chip_UART_TxCmd(LPC_UART1, ENABLE);
}

/* Sends a character on the UART */
static void board_uart_out_ch(char ch)
{
	while (Chip_UART_SendByte(LPC_UART1, (uint8_t) ch) == ERROR) {}
}

/* Sends a string on the UART */
static void board_uart_out_string(char *str)
{
	while (*str != '\0') {
		board_uart_out_ch(*str++);
	}
}
#endif

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Error processing function: stop with dying message
 * @param	rc	: FatFs return value
 * @return	Nothing
 */
void die(FRESULT rc)
{
	DEBUGOUT("Failed with rc=%u.\n", rc);
//	for (;; ) {}
}

/**
 * @brief	System tick interrupt handler
 * @return	Nothing
 */
void SysTick_Handler(void)
{
	u32Milliseconds += 10;
}

/**
 * @brief	SDIO controller interrupt handler
 * @return	Nothing
 */
void SDIO_IRQHandler(void)
{
	/* All SD based register handling is done in the callback
	   function. The SDIO interrupt is not enabled as part of this
	   driver and needs to be enabled/disabled in the callbacks or
	   application as needed. This is to allow flexibility with IRQ
	   handling for applicaitons and RTOSes. */
	/* Set wait exit flag to tell wait function we are ready. In an RTOS,
	   this would trigger wakeup of a thread waiting for the IRQ. */
	NVIC_DisableIRQ(SDIO_IRQn);
	sdio_wait_exit = 1;
}

/**
 * @brief	RIT interrupt handler sub-routine
 * @return	Nothing
 */
void RIT_IRQHandler(void)
{
	/* Clear interrupt flag */
	Chip_RIT_ClearInt();

	/* Disk timer function (100Hz) */
	disk_timerproc();
	if (timerCntms) {
		timerCntms--;
	}
}

/**
 * @brief	User Provided Timer Function for FatFs module
 * @return	Nothing
 * This is a real time clock service to be called from FatFs module.
 * Any valid time must be returned even if the system does not support a real time clock.
 * This is not required in read-only configuration.
 */
DWORD get_fattime()
{
	RTC rtc;

	/* Get local time */
	rtc_gettime(&rtc);

	/* Pack date and time into a DWORD variable */
	return ((DWORD) (rtc.year - 1980) << 25)
		   | ((DWORD) rtc.month << 21)
		   | ((DWORD) rtc.mday << 16)
		   | ((DWORD) rtc.hour << 11)
		   | ((DWORD) rtc.min << 5)
		   | ((DWORD) rtc.sec >> 1);
}

/**
 * @brief	Main routine for SDMMC example
 * @return	Nothing
 */
int main(void)
{
	FRESULT rc;		/* Result code */
	DIR dir;		/* Directory object */
	FILINFO fno;	/* File information object */
	UINT bw, br, i;
	char debugBuf[64];

	Board_Init();
#ifdef BOARD_HITEX_EVA_18504350
	board_uart_debug_init();
#endif

	App_SDMMC_Init();

	/* Initialize Repitetive Timer */
	App_Timer_Init();

	debugstr("\n\rHello NXP Semiconductors\n\rSDIO demo");

	/* Initialize RTC timer */
	rtc_initialize();

	NVIC_DisableIRQ(SDIO_IRQn);
	/* Enable SD/MMC Interrupt */
	NVIC_EnableIRQ(SDIO_IRQn);
	/* Enable RIT interrupt */
	NVIC_EnableIRQ(RITIMER_IRQn);

	f_mount(0, &Fatfs);		/* Register volume work area (never fails) */

	debugstr("\n\rOpen an existing file (message.txt).\n\r");

	rc = f_open(&Fil, "MESSAGE.TXT", FA_READ);
	if (rc) {
		die(rc);
	}

	for (;; ) {
		/* Read a chunk of file */
		rc = f_read(&Fil, Buff, sizeof Buff, &br);
		if (rc || !br) {
			break;					/* Error or end of file */
		}
		for (i = 0; i < br; i++)	/* Type the data */
#ifdef BOARD_HITEX_EVA_18504350
		{board_uart_out_ch((char) Buff[i]); }
#else
		{DEBUGOUT("%c", Buff[i]); }
#endif
	}
	if (rc) {
		die(rc);
	}

	debugstr("\n\rClose the file.\n");
	rc = f_close(&Fil);
	if (rc) {
		die(rc);
	}

	debugstr("\n\rCreate a new file (hello.txt).\n");
	rc = f_open(&Fil, "HELLO.TXT", FA_WRITE | FA_CREATE_ALWAYS);
	if (rc) {
		die(rc);
	}

	debugstr("\n\rWrite a text data. (Hello world!)\n");

	rc = f_write(&Fil, "Hello world!\r\n", 14, &bw);
	if (rc) {
		die(rc);
	}

	sprintf(debugBuf, "%u bytes written.\n", bw);
	debugstr(debugBuf);

	debugstr("\n\rClose the file.\n");
	rc = f_close(&Fil);
	if (rc) {
		die(rc);
	}

	debugstr("\n\rOpen root directory.\n");
	rc = f_opendir(&dir, "");
	if (rc) {
		die(rc);
	}

	debugstr("\n\rDirectory listing...\n\r");
	for (;; ) {
		/* Read a directory item */
		rc = f_readdir(&dir, &fno);
		if (rc || !fno.fname[0]) {
			break;					/* Error or end of dir */
		}
		if (fno.fattrib & AM_DIR) {
			sprintf(debugBuf, "   <dir>  %s\n\r", fno.fname);
		}
		else {
			sprintf(debugBuf, "   %8lu  %s\n\r", fno.fsize, fno.fname);
		}
		debugstr(debugBuf);
	}
	if (rc) {
		die(rc);
	}

	debugstr("\n\rTest completed.\n");
	for (;; ) {}
}

/**
 * @}
 */
