/*
 * @brief HAL USB functions for the LPC18xx microcontrollers
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#if defined(__LPC18XX__) || defined(__LPC43XX__)

#include "../HAL_LPC.h"
#include "../../../USBTask.h"

#if defined(USB_CAN_BE_DEVICE)
#include "../../../Device.h"

void HAL_USBConnect(uint8_t corenum, uint32_t con)
{
#if defined(USB_DEVICE_ROM_DRIVER)
	if (con) {
		USBD_API->hw->Connect(UsbHandle, 1);
	}
	else {
		USBD_API->hw->Connect(UsbHandle, 0);
	}
#else
	if (con) {
		USB_REG(corenum)->USBCMD_D |= (1 << 0);
	}
	else {
		USB_REG(corenum)->USBCMD_D &= ~(1 << 0);
	}
#endif
}

#endif

/* Assumed Xtal Frequency */
#define XTAL_FREQ   12000000
#define USB1_FREQ 60000000

void HAL_USBInit(uint8_t corenum)
{
	if (corenum) {
		uint32_t core_freq;
#if defined(__LPC43XX__)
		Chip_Clock_SetupMainPLLHz(CLKIN_CRYSTAL, CRYSTAL_MAIN_FREQ_IN, 180000000, 180000000);
#endif
		core_freq = Chip_Clock_GetMainPLLHz();
		/* connect DIVD to PLL1, need a condition for calculate div factor */
		LPC_CGU->IDIV_CTRL[CLK_IDIV_D] = (9 << 24) | (1 << 11) | ((core_freq / USB1_FREQ - 1) << 2);

		LPC_CGU->BASE_CLK[CLK_BASE_USB1] =  (0x0F << 24) | (1 << 11);	/* connect DIVD to base USB1 */

		/* Turn on the phy */
		LPC_CREG->CREG0 &= ~(1 << 5);
		#if defined(USB_CAN_BE_HOST)
		/* enable USB1_DP and USB1_DN on chip FS phy */
		LPC_SCU->SFSUSB = 0x16;
		#endif
		#if defined(USB_CAN_BE_DEVICE)
		/* enable USB1_DP and USB1_DN on chip FS phy */
		LPC_SCU->SFSUSB = 0x12;
		#endif
		LPC_USB1->PORTSC1_D |= (1 << 24);
	}
	else {
		/* Set up USB0 clock */
		/* Disable PLL first */
		LPC_CGU->PLL0USB_CTRL |= 1;
		/* the usb core require output clock = 480MHz */
		if (XTAL_FREQ == 12000000) {
			LPC_CGU->PLL0USB_MDIV = 0x06167FFA;
		}
		else if (XTAL_FREQ == 24000000) {
			LPC_CGU->PLL0USB_MDIV = 0x030C00FF;
		}
		else if (XTAL_FREQ == 48000000) {
			LPC_CGU->PLL0USB_MDIV = 0x02060007;
		}
		else {LPC_CGU->PLL0USB_MDIV = 0x06167FFA; }
		LPC_CGU->PLL0USB_NP_DIV = 0;
		LPC_CGU->PLL0USB_CTRL = 0x601C;
		LPC_CGU->PLL0USB_CTRL |= ((6 << 24) | (1 << 11));
		/* Enable PLL after all setting is done */
		LPC_CGU->PLL0USB_CTRL &= ~1;
		while ((LPC_CGU->PLL0USB_STAT & 1) == 0x0) ;
		/* Turn on the phy */
		LPC_CREG->CREG0 &= ~(1 << 5);
	}

#if defined(USB_CAN_BE_DEVICE) && (!defined(USB_DEVICE_ROM_DRIVER))
	/* reset the controller */
	USB_REG(corenum)->USBCMD_D = USBCMD_D_Reset;
	/* wait for reset to complete */
	while (USB_REG(corenum)->USBCMD_D & USBCMD_D_Reset) ;

	/* Program the controller to be the USB device controller */
	USB_REG(corenum)->USBMODE_D =   (0x2 << 0) /*| (1<<4)*//*| (1<<3)*/;
	if (corenum == 0) {
		/* set OTG transcever in proper state, device is present
		   on the port(CCS=1), port enable/disable status change(PES=1). */
		LPC_USB0->OTGSC = (1 << 3) | (1 << 0) /*| (1<<16)| (1<<24)| (1<<25)| (1<<26)| (1<<27)| (1<<28)| (1<<29)| (1<<30)*/;
		#if (USB_FORCED_FULLSPEED)
		LPC_USB0->PORTSC1_D |= (1 << 24);
		#endif
	}
	HAL_Reset();
#endif
}

void HAL_USBDeInit(uint8_t corenum)
{
	HAL_DisableUSBInterrupt(corenum);
	if (USB_CurrentMode == USB_MODE_Device) {
		#if defined(USB_CAN_BE_HOST)
		USB_REG(corenum)->USBSTS_H = 0xFFFFFFFF;				/* clear all current interrupts */
		USB_REG(corenum)->PORTSC1_H &= ~(1 << 12);					/* clear port power */
		USB_REG(corenum)->USBMODE_H =   (1 << 0);				/* set USB mode reserve */
		#endif
	}
	else if (USB_CurrentMode == USB_MODE_Host) {
		#if defined(USB_CAN_BE_DEVICE)
		/* Clear all pending interrupts */
		USB_REG(corenum)->USBSTS_D   = 0xFFFFFFFF;
		USB_REG(corenum)->ENDPTNAK   = 0xFFFFFFFF;
		USB_REG(corenum)->ENDPTNAKEN = 0;
		USB_REG(corenum)->ENDPTSETUPSTAT = USB_REG(corenum)->ENDPTSETUPSTAT;
		USB_REG(corenum)->ENDPTCOMPLETE  = USB_REG(corenum)->ENDPTCOMPLETE;
		while (USB_REG(corenum)->ENDPTPRIME) ;						/* Wait until all bits are 0 */
		USB_REG(corenum)->ENDPTFLUSH = 0xFFFFFFFF;
		while (USB_REG(corenum)->ENDPTFLUSH) ;		/* Wait until all bits are 0 */
		#endif
	}
}

void HAL_EnableUSBInterrupt(uint8_t corenum)
{
	NVIC_EnableIRQ((corenum) ? USB1_IRQn : USB0_IRQn);	//  enable USB interrupts
}

void HAL_DisableUSBInterrupt(uint8_t corenum)
{
	NVIC_DisableIRQ((corenum) ? USB1_IRQn : USB0_IRQn);	//  disable USB interrupts
}

void USB0_IRQHandler(void)
{
	if (USB_CurrentMode == USB_MODE_Host) {
		#ifdef USB_CAN_BE_HOST
		HcdIrqHandler(0);
		#endif
	}
	if (USB_CurrentMode == USB_MODE_Device) {
		#ifdef USB_CAN_BE_DEVICE
			#ifdef USB_DEVICE_ROM_DRIVER
		UsbdRom_IrqHandler();
			#else
		DcdIrqHandler(0);
			#endif
		#endif
	}
}

void USB1_IRQHandler(void)
{
	if (USB_CurrentMode == USB_MODE_Host) {
		#ifdef USB_CAN_BE_HOST
		HcdIrqHandler(1);
		#endif
	}
	if (USB_CurrentMode == USB_MODE_Device) {
		#ifdef USB_CAN_BE_DEVICE
			#ifdef USB_DEVICE_ROM_DRIVER
		UsbdRom_IrqHandler();
			#else
		DcdIrqHandler(1);
			#endif
		#endif
	}
}

#endif /*__LPC18XX__*/
